"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parsePath = parsePath;
exports.resolveImage = resolveImage;

var _promises = _interopRequireDefault(require("fs/promises"));

var _mime = require("mime");

var _path = _interopRequireDefault(require("path"));

var _Converter = require("./Converter");

var _ImageSizeResolver = require("./ImageSizeResolver");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function parsePath(resourcePath) {
  var _match$;

  // https://regexr.com/54acj
  const match = resourcePath.match(/^(?:(.*)[\\/])?([^\\/]+?)(?:@([0-9])x)?\.(.+)$/);

  if (!match || !match[2] || !match[4]) {
    throw new Error(`Unable to parse resource ${resourcePath}.`);
  }

  return {
    dir: (_match$ = match[1]) !== null && _match$ !== void 0 ? _match$ : '',
    name: match[2],
    scale: match[3] != null ? parseFloat(match[3]) : 1,
    ext: match[4]
  };
}

async function resolveImage(resourcePath, resourceContent, scales, formats, emitFileCallback) {
  const fileData = parsePath(resourcePath);
  const type = (0, _mime.getType)(fileData.ext);

  if (type == null) {
    throw new Error(`Unable to parse mime type for ${resourcePath}.`);
  }

  const generateModernFormats = type === 'image/png' || type === 'image/jpeg';

  const getFilePath = scale => {
    const suffix = scale === 1 ? '' : `@${scale}x`;
    return _path.default.join(fileData.dir, `${fileData.name}${suffix}.${fileData.ext}`);
  };

  const addFile = async (scale, content) => {
    const filePath = getFilePath(scale);
    emitFileCallback({
      uri: filePath,
      scale,
      type
    }, content);

    if (generateModernFormats) {
      if (formats.avif) {
        emitFileCallback({
          uri: filePath.replace(/\.png|\.jpe?g/, '.avif'),
          scale,
          type: 'image/avif'
        }, await (0, _Converter.convertToAvif)(content));
      }

      if (formats.webp) {
        emitFileCallback({
          uri: filePath.replace(/\.png|\.jpe?g/, '.webp'),
          scale,
          type: 'image/webp'
        }, await (0, _Converter.convertToWebp)(content));
      }
    }
  }; // Original file will be passed as `resourceContent`.


  await addFile(fileData.scale, resourceContent);
  const size = (0, _ImageSizeResolver.imageSize)(resourceContent, fileData.scale); // Find other files available for scales.

  const missingScales = scales.filter(s => s !== fileData.scale);

  for (const scale of missingScales) {
    try {
      const filePath = getFilePath(scale);
      const stats = await _promises.default.stat(filePath);

      if (stats.isFile()) {
        const content = await _promises.default.readFile(filePath);
        await addFile(scale, content);
      }
    } catch (e) {// Do nothing
    }
  }

  return size;
}
//# sourceMappingURL=ImageResolver.js.map